@@ -25,6 +25,10 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
+#ifdef _WIN32
+#include "win32dep.h"
+#endif
+
 #include "pixmaps/ok.xpm"
 #include "pixmaps/cancel.xpm"
 #include "pixmaps/tb_forward.xpm"
@@ -32,11 +36,12 @@
 GSList *protocols = NULL;
 
 GtkWidget *protomenu = NULL;
+int prpl_accounts[PROTO_UNTAKEN];
 
 struct _prompt {
 	GtkWidget *window;
 	GtkWidget *entry;
-	void (*doit)(void *, char *);
+	void (*doit)(void *, const char *);
 	void (*dont)(void *);
 	void *data;
 };
@@ -56,52 +61,69 @@ struct prpl *find_prpl(int prot)
 	return NULL;
 }
 
-static gint proto_compare(struct prpl *a, struct prpl *b)
+gint proto_compare(struct prpl *a, struct prpl *b)
 {
 	/* neg if a before b, 0 if equal, pos if a after b */
 	return a->protocol - b->protocol;
 }
 
-void load_protocol(proto_init pi, int size)
+#ifdef GAIM_PLUGINS
+gboolean load_prpl(struct prpl *p)
 {
-	struct prpl *p;
-	struct prpl *old;
-	if (size != sizeof(struct prpl)) {
-		do_error_dialog(_("You have attempted to load a protocol which was not compiled"
-				  " from the same version of the source as this application was."
-				  " Unfortunately, because it is not the same version I cannot"
-				  " safely tell you which one it was. Needless to say, it was not"
-				  " successfully loaded."), _("Protocol Error"));
-		return;
+	char *(*gaim_prpl_init)(struct prpl *);
+	debug_printf("Loading protocol %d\n", p->protocol);
+
+	if (!p->plug)
+		return TRUE;
+	
+	p->plug->handle = g_module_open(p->plug->path, 0);
+	if (!p->plug->handle) {
+		debug_printf("%s is unloadable: %s\n", p->plug->path, g_module_error());
+		return TRUE;
 	}
-	p = g_new0(struct prpl, 1);
+
+	if (!g_module_symbol(p->plug->handle, "gaim_prpl_init", (gpointer *)&gaim_prpl_init)) {
+		return TRUE;
+	}
+
+	gaim_prpl_init(p);
+	return FALSE;
+}
+#endif
+
+/* This is used only by static protocols */
+void load_protocol(proto_init pi)
+{
+	struct prpl *p = g_new0(struct prpl, 1);
+
+	if (p->protocol == PROTO_ICQ) 
+		do_error_dialog(_("ICQ Protocol detected."),
+				_("Gaim has loaded the ICQ plugin.  This plugin has been deprecated. "
+				  "As such, it was probably not compiled from the same version of the "
+				  "source as this application was, and cannot be guaranteed to work.  "
+				  "It is reccomended that you use the AIM/ICQ protocol to connect to ICQ"),
+				GAIM_WARNING);
 	pi(p);
-	if ((old = find_prpl(p->protocol)) != NULL)
-		unload_protocol(old);
 	protocols = g_slist_insert_sorted(protocols, p, (GCompareFunc)proto_compare);
 	regenerate_user_list();
 }
 
 void unload_protocol(struct prpl *p)
 {
-	GSList *c = connections;
-	struct gaim_connection *g;
+	GList *c;
+	struct proto_user_opt *puo;
+	if (p->name)
+		g_free(p->name);
+	c = p->user_opts;
 	while (c) {
-		g = (struct gaim_connection *)c->data;
-		if (g->prpl == p) {
-			char buf[256];
-			g_snprintf(buf, sizeof buf, _("%s was using %s, which got removed."
-						      " %s is now offline."), g->username,
-				   p->name(), g->username);
-			do_error_dialog(buf, _("Disconnect"));
-			signoff(g);
-			c = connections;
-		} else
-			c = c->next;
+		puo = c->data;
+		g_free(puo->label);
+		g_free(puo->def);
+		g_free(puo);
+		c = c->next;
 	}
-	protocols = g_slist_remove(protocols, p);
-	g_free(p);
-	regenerate_user_list();
+	g_list_free(p->user_opts);
+	p->user_opts = NULL;
 }
 
 STATIC_PROTO_INIT
@@ -119,46 +141,85 @@ static void rem_win(GtkWidget *a, GtkWidget *b)
 	gtk_widget_destroy(b);
 }
 
-void do_ask_dialog(const char *text, void *data, void *doit, void *dont)
+struct doaskstruct {
+	void (*yesfunc)(gpointer);
+	void (*nofunc)(gpointer);
+	gpointer data;
+};
+
+static void do_ask_callback(GtkDialog *d, gint resp, struct doaskstruct *doask)
+{
+	switch (resp) 
+		{
+		case GTK_RESPONSE_YES:
+			if (doask->yesfunc)
+				doask->yesfunc(doask->data);
+			break;
+		case GTK_RESPONSE_NO:
+		case GTK_RESPONSE_DELETE_EVENT:
+			if (doask->nofunc)
+				doask->nofunc(doask->data);
+			break;
+		}
+	g_free(doask);
+	gtk_widget_destroy(GTK_WIDGET(d));
+}
+
+#define STOCK_ITEMIZE(r, l)       if (!strcmp(r,yestext))  \
+                                           yestext = l;    \
+                                   if (!strcmp(r,notext))  \
+                                           notext = l;     
+
+void do_ask_dialog(const char *prim, const char *sec, void *data, char *yestext, void *doit, char *notext, void *dont)
 {
 	GtkWidget *window;
-	GtkWidget *vbox;
-	GtkWidget *label;
 	GtkWidget *hbox;
-	GtkWidget *button;
+	GtkWidget *label;
+	char labeltext[1024 * 2];
+	char *filename = g_build_filename(DATADIR, "pixmaps", "gaim", "dialogs", "gaim_question.png", NULL);
+	GtkWidget *img = gtk_image_new_from_file(filename);
+	struct doaskstruct *doask = g_new0(struct doaskstruct, 1);
 
-	GAIM_DIALOG(window);
-	gtk_window_set_wmclass(GTK_WINDOW(window), "accept", "Gaim");
-	gtk_window_set_policy(GTK_WINDOW(window), FALSE, TRUE, TRUE);
-	gtk_window_set_title(GTK_WINDOW(window), _("Accept?"));
-	gtk_widget_realize(window);
-	aol_icon(window->window);
-	if (dont)
-		gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(dont), data);
+	doask->yesfunc = doit;
+	doask->nofunc  = dont;
+	doask->data    = data;
 
-	vbox = gtk_vbox_new(FALSE, 5);
-	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
-	gtk_container_add(GTK_CONTAINER(window), vbox);
+	g_free(filename);
+	gtk_misc_set_alignment(GTK_MISC(img), 0, 0);
 
-	label = gtk_label_new(text);
-	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
+	/* This is ugly.  GTK Stock items will take a button with a label "gtk-cancel" and turn it into a 
+	 * Cancel button with a Cancel icon and whatnot.  We want to avoid using anything gtk in the prpls
+	 * so we replace "Cancel" with "gtk-cancel" right here. */
+	STOCK_ITEMIZE("Add", GTK_STOCK_ADD);
+	STOCK_ITEMIZE("Apply", GTK_STOCK_APPLY);
+	STOCK_ITEMIZE("Cancel", GTK_STOCK_CANCEL);
+	STOCK_ITEMIZE("Close", GTK_STOCK_CLOSE);
+	STOCK_ITEMIZE("Delete", GTK_STOCK_DELETE);
+	STOCK_ITEMIZE("Remove", GTK_STOCK_REMOVE);
+	STOCK_ITEMIZE("Yes", GTK_STOCK_YES);
+	STOCK_ITEMIZE("No", GTK_STOCK_NO);
+
+	window = gtk_dialog_new_with_buttons("", NULL, GTK_DIALOG_MODAL, notext, GTK_RESPONSE_NO, yestext, GTK_RESPONSE_YES, NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG(window), GTK_RESPONSE_YES);
+	g_signal_connect(G_OBJECT(window), "response", G_CALLBACK(do_ask_callback), doask);
+	
+	gtk_container_set_border_width (GTK_CONTAINER(window), 6);
+	gtk_window_set_resizable(GTK_WINDOW(window), FALSE);
+	gtk_dialog_set_has_separator(GTK_DIALOG(window), FALSE);
+	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(window)->vbox), 12);
+	gtk_container_set_border_width (GTK_CONTAINER(GTK_DIALOG(window)->vbox), 6);
+
+	hbox = gtk_hbox_new(FALSE, 12);
+	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(window)->vbox), hbox);
+	gtk_box_pack_start(GTK_BOX(hbox), img, FALSE, FALSE, 0);
+	
+	g_snprintf(labeltext, sizeof(labeltext), "<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s", prim, sec ? sec : "");
+	label = gtk_label_new(NULL);
+	gtk_label_set_markup(GTK_LABEL(label), labeltext);
 	gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-
-	hbox = gtk_hbox_new(FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
-
-	button = picture_button(window, _("Cancel"), cancel_xpm);
-	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(des_win), window);
-
-	button = picture_button(window, _("Accept"), ok_xpm);
-	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
-	if (dont)
-		gtk_object_set_user_data(GTK_OBJECT(button), data);
-	if (doit)
-		gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(doit), data);
-	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(rem_win), window);
-
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+	
 	gtk_widget_show_all(window);
 }
 
@@ -199,7 +260,6 @@ void do_prompt_dialog(const char *text, const char *def, void *data, void *doit,
 	gtk_window_set_title(GTK_WINDOW(window), _("Gaim - Prompt"));
 	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(des_prompt), p);
 	gtk_widget_realize(window);
-	aol_icon(window->window);
 
 	vbox = gtk_vbox_new(FALSE, 5);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
@@ -311,7 +371,7 @@ void do_proto_menu()
 				continue;
 			}
 
-			g_snprintf(buf, sizeof(buf), "%s (%s)", gc->username, gc->prpl->name());
+			g_snprintf(buf, sizeof(buf), "%s (%s)", gc->username, gc->prpl->name);
 			menuitem = gtk_menu_item_new_with_label(buf);
 			gtk_menu_append(GTK_MENU(protomenu), menuitem);
 			gtk_widget_show(menuitem);
@@ -396,9 +456,12 @@ void connection_has_mail(struct gaim_connection *gc, int count, const char *from
 		mailnots = g_slist_append(mailnots, mn);
 	}
 
-	if (count < 0 && from && subject) {
-		g_snprintf(buf, sizeof buf, "%s has mail from %s: %s", gc->username, from, subject);
-	} else if (count) {
+	if (count < 0) {
+		if (from && subject)
+			g_snprintf(buf, sizeof buf, "%s has mail from %s: %s", gc->username, from, subject);
+		else
+			g_snprintf(buf, sizeof buf, "%s has new mail.", gc->username);
+	} else if (count > 0) {
 		g_snprintf(buf, sizeof buf, "%s has %d new message%s.",
 			   gc->username, count, count == 1 ? "" : "s");
 	} else if (mn->email_win) {
@@ -419,7 +482,6 @@ void connection_has_mail(struct gaim_connection *gc, int count, const char *from
 	gtk_window_set_title(GTK_WINDOW(mn->email_win), _("Gaim - New Mail"));
 	gtk_signal_connect(GTK_OBJECT(mn->email_win), "destroy", GTK_SIGNAL_FUNC(des_email_win), mn);
 	gtk_widget_realize(mn->email_win);
-	aol_icon(mn->email_win->window);
 
 	vbox = gtk_vbox_new(FALSE, 5);
 	gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
@@ -428,7 +490,8 @@ void connection_has_mail(struct gaim_connection *gc, int count, const char *from
 
 	mn->email_label = gtk_label_new(buf);
 	gtk_label_set_text(GTK_LABEL(mn->email_label), buf);
-	gtk_box_pack_start(GTK_BOX(vbox), mn->email_label, 0, 0, 5);
+	gtk_label_set_line_wrap(GTK_LABEL(mn->email_label), TRUE);
+	gtk_box_pack_start(GTK_BOX(vbox), mn->email_label, FALSE, TRUE, 5);
 	gtk_widget_show(mn->email_label);
 
 	hbox = gtk_hbox_new(FALSE, 5);
@@ -470,9 +533,15 @@ static gint find_icon_data(gconstpointer a, gconstpointer b)
 
 void set_icon_data(struct gaim_connection *gc, char *who, void *data, int len)
 {
-	struct icon_data tmp = { gc, normalize(who), NULL, 0 };
-	GList *l = g_list_find_custom(icons, &tmp, find_icon_data);
-	struct icon_data *id = l ? l->data : NULL;
+	struct icon_data tmp;
+	GList *l;
+	struct icon_data *id;
+	tmp.gc = gc;
+	tmp.who = normalize(who);
+	tmp.data=NULL;
+        tmp.len = 0;
+	l = g_list_find_custom(icons, &tmp, find_icon_data);
+	id = l ? l->data : NULL;
 
 	if (id) {
 		g_free(id->data);
@@ -537,7 +606,7 @@ struct got_add {
 	char *alias;
 };
 
-static void dont_add(gpointer x, struct got_add *ga)
+static void dont_add(struct got_add *ga)
 {
 	g_free(ga->who);
 	if (ga->alias)
@@ -545,10 +614,11 @@ static void dont_add(gpointer x, struct got_add *ga)
 	g_free(ga);
 }
 
-static void do_add(gpointer x, struct got_add *ga)
+static void do_add(struct got_add *ga)
 {
 	if (g_slist_find(connections, ga->gc))
 		show_add_buddy(ga->gc, ga->who, NULL, ga->alias);
+	dont_add(ga);
 }
 
 void show_got_added(struct gaim_connection *gc, const char *id,
@@ -571,9 +641,9 @@ void show_got_added(struct gaim_connection *gc, const char *id,
 		   msg ? msg : "",
 		   find_buddy(gc, ga->who) ? "" : _("\n\nDo you wish to add him or her to your buddy list?"));
 	if (find_buddy(gc, ga->who))
-		do_error_dialog(buf, "Added to List");
+		do_error_dialog(buf, NULL, GAIM_INFO);
 	else
-		do_ask_dialog(buf, ga, do_add, dont_add);
+		do_ask_dialog(buf, NULL, ga, _("Add"), do_add, _("Cancel"), dont_add);
 }
 
 static GtkWidget *regdlg = NULL;
@@ -652,7 +722,6 @@ void register_dialog()
 	gtk_window_set_title(GTK_WINDOW(regdlg), _("Gaim - Registration"));
 	gtk_window_set_wmclass(GTK_WINDOW(regdlg), "register", "Gaim");
 	gtk_widget_realize(regdlg);
-	aol_icon(regdlg->window);
 	gtk_signal_connect(GTK_OBJECT(regdlg), "destroy", GTK_SIGNAL_FUNC(delete_regdlg), NULL);
 
 	vbox = gtk_vbox_new(FALSE, 5);
